﻿using System;
using System.Windows;
using System.Windows.Controls;
using ESRI.ArcGIS.Client;
using ESRI.ArcGIS.Client.Symbols;
using ESRI.ArcGIS.Client.Tasks;
using Microsoft.Phone.Controls;

namespace ArcGISWindowsPhoneSDK
{
    public partial class SpatialQuery : PhoneApplicationPage
    {
        private Draw MyDrawSurface;
        private Symbol _activeSymbol = null;

        public SpatialQuery()
        {
            InitializeComponent();

            MyDrawSurface = new Draw(MyMap)
            {
                LineSymbol = LayoutRoot.Resources["DefaultLineSymbol"] as SimpleLineSymbol,
                FillSymbol = LayoutRoot.Resources["DefaultFillSymbol"] as FillSymbol
            };
            MyDrawSurface.DrawComplete += MyDrawSurface_DrawComplete;
            MyDrawSurface.DrawMode = DrawMode.Point;
            MyDrawSurface.IsEnabled = true;
        }

        private void DataListBox_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            foreach (Graphic g in e.AddedItems)
                g.Select();

            foreach (Graphic g in e.RemovedItems)
                g.UnSelect();
        }

        private void MyDrawSurface_DrawComplete(object sender, ESRI.ArcGIS.Client.DrawEventArgs args)
        {
            GraphicsLayer selectionGraphicslayer = MyMap.Layers["MySelectionGraphicsLayer"] as GraphicsLayer;
            selectionGraphicslayer.ClearGraphics();

            QueryTask queryTask = new QueryTask("http://sampleserver1.arcgisonline.com/ArcGIS/rest/services/Demographics/ESRI_Census_USA/MapServer/5");
            queryTask.ExecuteCompleted += QueryTask_ExecuteCompleted;
            queryTask.Failed += QueryTask_Failed;

            Query query = new ESRI.ArcGIS.Client.Tasks.Query()
            {
                Geometry = args.Geometry,
                OutSpatialReference = MyMap.SpatialReference,
                ReturnGeometry = true
            };

            // Specify fields to return from query
            query.OutFields.AddRange(new string[] { "STATE_NAME", "POP2000", "POP2007" });
 
            queryTask.ExecuteAsync(query);
        }

        private void QueryTask_ExecuteCompleted(object sender, ESRI.ArcGIS.Client.Tasks.QueryEventArgs args)
        {
            FeatureSet featureSet = args.FeatureSet;
            if (featureSet == null || featureSet.Features.Count < 1)
            {
                MessageBox.Show("No features returned from query");
                return;
            }

            GraphicsLayer graphicsLayer = MyMap.Layers["MySelectionGraphicsLayer"] as GraphicsLayer;

            if (featureSet != null && featureSet.Features.Count > 0)
            {
                foreach (Graphic feature in featureSet.Features)
                {
                    feature.Symbol = LayoutRoot.Resources["ResultsFillSymbol"] as FillSymbol;
                    graphicsLayer.Graphics.Insert(0, feature);
                }
            }

            DataListBox.ItemsSource = featureSet;
            MyDrawSurface.IsEnabled = false;
        }

        private void QueryTask_Failed(object sender, TaskFailedEventArgs args)
        {
            MessageBox.Show("Query failed: " + args.Error);
        }

        private void DrawPointButton_Click(object sender, EventArgs e)
        {
            MyDrawSurface.DrawMode = DrawMode.Point;
            MyDrawSurface.IsEnabled = true;
        }

        private void DrawPolylineButton_Click(object sender, EventArgs e)
        {
            MyDrawSurface.DrawMode = DrawMode.Polyline;
            _activeSymbol = LayoutRoot.Resources["DefaultLineSymbol"] as Symbol;
            MyDrawSurface.IsEnabled = true;
        }

        private void DrawPolygonButton_Click(object sender, EventArgs e)
        {
            MyDrawSurface.DrawMode = DrawMode.Polygon;
            _activeSymbol = LayoutRoot.Resources["DefaultFillSymbol"] as Symbol;
             MyDrawSurface.IsEnabled = true;
        }

        private void DrawRectangleButton_Click(object sender, EventArgs e)
        {
            MyDrawSurface.DrawMode = DrawMode.Rectangle;
            _activeSymbol = LayoutRoot.Resources["DefaultFillSymbol"] as Symbol;
            MyDrawSurface.IsEnabled = true;
        }

        private void ClearGraphicsMenuItem_Click(object sender, EventArgs e)
        {
            MyDrawSurface.DrawMode = DrawMode.None;
            GraphicsLayer graphicsLayer = MyMap.Layers["MySelectionGraphicsLayer"] as GraphicsLayer;
            graphicsLayer.ClearGraphics();
            DataListBox.ItemsSource = null;
        }
 
        private void GraphicsLayer_MouseLeftButtonDown(object sender, GraphicMouseButtonEventArgs e)
        {
            DataListBox.Focus();
            DataListBox.SelectedItem = e.Graphic;
            DataListBox.ScrollIntoView(DataListBox.SelectedItem);
        }
    }
}